#!/usr/bin/env python
# -*- coding: utf-8 -*-

''' Iterates over a bunch of .jpg or .cr2 files and matches 
DateTimeOriginal from Exif tag to DateTime in a csv log 
of a GeigerMuellerCounter and writes its value to the UserComment
Exif tag in µS/h '''

import csv
import argparse
import pytz
import gpxpy
from functions import Radiation, Photo, Match, Exif

# SIFACTOR for GQ Geiger counters

# 300  series: 0.0065   µSv/h / CPM
# 320  series: 0.0065   µSv/h / CPM
# 500  series: 0.0065   µSv/h / CPM
# 500+ series: 0.0065   µSv/h / CPM for the first tube
# 600  series: 0.0065   µSv/h / CPM
# 600+ series: 0.002637 µSv/h / CPM

# Configure argument parser for cli options
parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter, 
                                 description='''A unix-tyle tool that 
    extracts GPS and/or radiation data from GPX/CSV files and writes
    them into the Exif tags of given photos.''')
parser.add_argument('-si', '--sifactor', type=float, default=0.0065, 
                    help='Factor to multiply recorded CPM with.')
parser.add_argument('-tz', '--timezone', type=str, metavar='Timezone', default='utc',
                    help='''Manually set timezone of CSV / and Photo timestamp,
                    defaults to UTC if omitted. This is useful, if the GPS-Logger
                    saves the time incl. timezone''')
parser.add_argument('-d', '--dry', action='store_true', 
                    help='Dry-run, do not actually write anything.')
parser.add_argument('csv', metavar='CSV', type=str, 
                    help='Geiger counter history file in CSV format.')
parser.add_argument('-g', '--gpx', metavar='GPX', type=str,
                    help='GPS track in GPX format')
parser.add_argument('photos', metavar='Photo', type=str, nargs='+', 
                    help='One or multiple photo image files to process.')
parser.add_argument('-o', '--outdir', type=str, default='.', 
                    help='Directory to output processed photos.')

args = parser.parse_args()

# Create timezone datetime object
local_timezone = pytz.timezone(args.timezone)

# Initialize two empty lists for comparison
radiation_list = []
position_list = []

# Import GeigerCounter log
with open(args.csv, "r") as f:
    csv = csv.reader(filter(lambda row: row[0] != '#', f), 
                     delimiter=',', skipinitialspace=True)

    for _, csv_raw_time, csv_raw_cpm, _ in csv:
        radiation = Radiation(csv_raw_time, csv_raw_cpm, local_timezone, args.sifactor)
        radiation_list.append(radiation)
    # close CSV file
    f.close()

# Import GPX track(s)print
if args.gpx is not None:
    gpx_file = open(args.gpx, 'r')
    gpx_reader = gpxpy.parse(gpx_file)
    #for waypoint in gpx_reader.waypoints:
    for track in gpx_reader.tracks:
        for segment in track.segments:
            for point in segment.points:
                point_aware_time = point.time.astimezone(local_timezone)
                #point_aware_time = point_naive_time.astimezone(local_timezone)                
                position = (point_aware_time, point.latitude, point.longitude,
                            point.elevation)
                position_list.append(position)

# Inform the user about what is going to happen
if args.dry is True:
    print('Not modifying anything. Just print what would happen without --dry')
else:
    if args.outdir == ".":
        print('Modifying photos in place (overwrite)')
    else:
        print('Modifying photos in', str(args.outdir), '(copy)')

for src_photo in args.photos:
    photo = Photo(src_photo, local_timezone, args.outdir, args.dry)
    print(photo.get_target_photo, photo.get_date)

    # Here the matching magic has to happen

    match = Match(photo.get_date, radiation_list, position_list)

    # Write exif data
    exif_tags = Exif(photo.get_target_photo, args.dry, match.radiation[1], 
                     match.position[1][1], match.position[1][2], match.position[1][3])
    print(exif_tags)
